@@ -59,6 +59,8 @@ on_iface_properties_clicked (GtkWidget *widget, gpointer data)
       dialog = GST_NETWORK_TOOL (tool)->dialog;
       connection_dialog_prepare (dialog, iface);
       gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), GTK_WINDOW (tool->main_dialog));
+
+      gst_dialog_add_edit_dialog (tool->main_dialog, dialog->dialog);
       gtk_widget_show (dialog->dialog);
     }
 }
@@ -118,6 +120,55 @@ on_bootproto_changed (GtkWidget *widget, gpointer data)
   g_free (method);
 }
 
+void
+on_ppp_type_changed (GtkWidget *widget, gpointer data)
+{
+  GstConnectionDialog *dialog;
+  gchar *type;
+
+  dialog = GST_NETWORK_TOOL (tool)->dialog;
+  type = connection_combo_get_value (GTK_COMBO_BOX (dialog->ppp_type_combo));
+
+  on_dialog_changed (widget, data);
+
+  if (!type)
+    return;
+
+  if (strcmp (type, "modem") == 0)
+    {
+      gtk_widget_show (dialog->modem_page);
+      gtk_widget_show (dialog->isp_frame);
+      gtk_widget_show (dialog->modem_settings_table);
+      gtk_widget_show (dialog->modem_isp_table);
+      gtk_widget_hide (dialog->pppoe_settings_table);
+      gtk_widget_hide (dialog->gprs_isp_table);
+    }
+  else if (strcmp (type, "isdn") == 0)
+    {
+      gtk_widget_show (dialog->isp_frame);
+      gtk_widget_show (dialog->modem_isp_table);
+      gtk_widget_hide (dialog->modem_page);
+      gtk_widget_hide (dialog->gprs_isp_table);
+    }
+  else if (strcmp (type, "pppoe") == 0)
+    {
+      gtk_widget_show (dialog->modem_page);
+      gtk_widget_show (dialog->pppoe_settings_table);
+      gtk_widget_show (dialog->modem_isp_table);
+      gtk_widget_hide (dialog->isp_frame);
+      gtk_widget_hide (dialog->modem_settings_table);
+    }
+  else if (strcmp (type, "gprs") == 0)
+    {
+      gtk_widget_show (dialog->modem_page);
+      gtk_widget_show (dialog->isp_frame);
+      gtk_widget_show (dialog->modem_settings_table);
+      gtk_widget_show (dialog->gprs_isp_table);
+      gtk_widget_hide (dialog->modem_isp_table);
+      gtk_widget_hide (dialog->pppoe_settings_table);
+    }
+}
+
 static void
 cancel_connection_dialog (GstTool *tool)
 {
@@ -198,6 +249,7 @@ on_connection_response (GtkWidget *widget,
 			gint       response,
 			gpointer   data)
 {
+  GstConnectionDialog *connection_dialog;
   GstDialog *dialog = GST_DIALOG (data);
   GstTool *tool = gst_dialog_get_tool (dialog);
 
@@ -205,6 +257,9 @@ on_connection_response (GtkWidget *widget,
     accept_connection_dialog (tool);
   else
     cancel_connection_dialog (tool);
+
+  connection_dialog = GST_NETWORK_TOOL (tool)->dialog;
+  gst_dialog_remove_edit_dialog (tool->main_dialog, connection_dialog->dialog);
 }
 
 void
@@ -332,7 +387,7 @@ on_table_popup_menu (GtkWidget *widget, gpointer data)
 void
 on_host_aliases_add_clicked (GtkWidget *widget, gpointer data)
 {
-  host_aliases_run_dialog (NULL);
+  host_aliases_run_dialog (GST_NETWORK_TOOL (tool), NULL);
 }
 
 void
@@ -347,7 +402,7 @@ on_host_aliases_properties_clicked (GtkWidget *widget, gpointer data)
   selection = gtk_tree_view_get_selection (list);
 
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    host_aliases_run_dialog (&iter);
+    host_aliases_run_dialog (GST_NETWORK_TOOL (tool), &iter);
 }
 
 void
